//*************************************************
// CORSO PROGRAMMAZIONE PICMICRO
// appendice 1
// www.settorezero.com
//
// modulo: main.c
// autore: Bernardo Giovanni
// data: 02/02/10
// descrizione: test encoder in quadratura
// picmicro: PIC16F877A
// clock: 20MHz
//
//*************************************************

#define  XTAL_FREQ 20MHZ // questo  utilizzato dalle routine di ritardo contenute in Delay.C
#include <pic.h> // contiene i nomi mnemonici di registri e porte

// Fuses di configurazione
__CONFIG (HS & WDTDIS & PWRTEN & BORDIS & LVPDIS & DUNPROT & WRTEN & DEBUGDIS & UNPROTECT);

#include "delay.c" // routine per ritardi
#include "settings.h" // settaggi del picmicro
#include "lcd.c"

// funzione principale, eseguita all'avvio del picmicro
void main(void)
	{
	BUZZER=0;
	suona=0;
	
	settings(); // eseguo la funzione settings contenuta nel file header settings.h, cos imposto le porte e i registri
	LCD_INIT(); // inizializzo l'LCD
	DelayMs(100); // piccolo ritardo iniziale
	LCDBL=1; // attivo la retroilluminazione sull' LCD
	LCD_CLEAR(); // ripulisco il display
	LCD_GOTO(1,1); // posiziono il cursore sull'LCD: riga1, colonna1
	LCD_PUTS("Test Encoder");
	LCD_GOTO(2,1);
	LCD_PUTS("Variabile:");
	LCD_PUTSN(variabile); // da notare che uso la funzione LCD_PUTSN che serve a mostrare sul display valori numerici con segno
	LCD_PUTS("% ");
	
	while(1) // eseguo un ciclo finito
        {
		LCD_GOTO(2,11);
		LCD_PUTSN(variabile);
		LCD_PUTS("% ");
		
		// modalit incremento e sono passati 50 millisecondi
		if (mode==INCR && ABcounter>50)
			{
			variabile++; // incremento la variabile di controllo
			suona=1; // setto il flag per far suonare il cicalino
			if (variabile>MAXVAR) // riporto la variabile al valore massimo se lo superiamo
				{
				variabile=MAXVAR;
				}
			mode=0; // nessuna modalit
			ABcounter=0; // resetto contatore
			INTE=1; // riattivo interrupt su RB0/INT
			}
		
		// modalit decremento e sono passati 50 millisecondi
		else if (mode==DECR && ABcounter>50)
			{
			variabile--; // incremento la variabile di controllo
			suona=1; // setto il flag per far suonare il cicalino
			if (variabile<MINVAR) // riporto la variabile al valore minimo se lo superiamo
				{
				variabile=MINVAR;
				}
			mode=0; // nessuna modalit
			ABcounter=0; // resetto contatore
			INTE=1; // riattivo interrupt su RB0/INT
			}
		
		if (suona) // devo suonare il cicalino?
			{
			for (char a=0; a<10; a++)
				{
				suona=0;
				DelayUs(200);
				BUZZER=1;
				DelayUs(200);
				BUZZER=0;
				}
			}
			
		}// Fine ciclo continuo
	} // Fine main
	
void interrupt ISR (void)
	{
	if (INTF) // L'interrupt  stato causato da un cambio di stato di RB0/INT ?
		{
		if (ENCODER_PULSE==ENCODER_DIR) // rotazione oraria
			{
			INTE=0; // disattivo l'interrupt su RB0/INT
			mode=INCR; // modalit incremento
			ABcounter=0; // resetto contatore
			}
		else // rotazione antioraria
			{
			INTE=0; // disattivo l'interrupt su RB0/INT
			mode=DECR; // modalit decremento
			ABcounter=0; // resetto contatore
			}
		INTF=0; // Resetto il flag interrupt su RB0/INT
		} // fine che interrupt verificatosi su RB0/INT
		
	if (T0IF) // L'interrupt  stato causato dall'overflow del timer 0 ?
		{
		TMR0=100;
		ABcounter++;
		if (ABcounter>250)
			{
			ABcounter=0;
			}
		T0IF=0; // resetto il flag di interrupt
		} // fine che interrupt verificatosi su TMR0
	} // fine interrupt service routine